@@ -561,6 +561,16 @@ class Sf:
         if self.debug:
             print("Extracted " + objectName + " successfully!")
         return sfdf
+    
+    def entity_access_query(self, entity_type):
+        # Retrieves information about which Profile or PermissionSet
+        # grants permission to which Setup Entity (i.e. ApexPage, ApexClass, TabSets, etc)
+        
+        entity_access = transform_sf_result_set_rec(self.sf.query_all(f"""
+        SELECT Id, Parent.Id, Parent.ProfileId, Parent.Profile.Name, Parent.Name,SetupEntityId,SetupEntityType FROM
+        SetupEntityAccess WHERE SetupEntityType = '{entity_type}'""")['records'])
+        # entity_access.drop(columns=['PermissionSet.Profile'], inplace=True)
+        return {entity_type: entity_access}
 
     ### ----------------------------------------------------------------------------------------------------
     ### Asynchronous Data Functions
@@ -601,15 +611,15 @@ class Sf:
         result_list = await asyncio.gather(*[async_delete(object_api= object_api, record_dataframe=batch) for batch in np.array_split(record_dataframe, batches)])
         return pd.concat(result_list).reset_index(drop=True)
     
-    def entity_access_query(self, entity_type):
-        # Retrieves information about which Profile or PermissionSet
-        # grants permission to which Setup Entity (i.e. ApexPage, ApexClass, TabSets, etc)
-        
-        entity_access = transform_sf_result_set_rec(self.sf.query_all(f"""
-        SELECT Id, Parent.Id, Parent.ProfileId, Parent.Profile.Name, Parent.Name,SetupEntityId,SetupEntityType FROM
-        SetupEntityAccess WHERE SetupEntityType = '{entity_type}'""")['records'])
-        # entity_access.drop(columns=['PermissionSet.Profile'], inplace=True)
-        return {entity_type: entity_access}
+    async def async_get_user_password_status(self, user_ids: list[str]) -> dict:
+        async_sf_api_call = async_wrap(self.sf_api_call)
+        response = await asyncio.gather(*[async_sf_api_call(f'/services/data/{sf.api_version}/sobjects/User/{user_id}/password', method='GET') for user_id in user_ids])
+        # Return a dict
+        response = {user_id: result for user_id, result in zip(user_ids, response)}
+        return response
+    
+    
+    
     ### ----------------------------------------------------------------------------------------------------
     ### Asynchronous Metadata Functions
     ### ----------------------------------------------------------------------------------------------------
@@ -1397,7 +1407,7 @@ class Sf:
     def tooling_query_all(self, query):
         done = False
         full_results = []
-        result = self.sf_api_call('/services/data/v52.0/tooling/query', method='get', parameters={'q': query})
+        result = self.sf_api_call(f'/services/data/v52.0/tooling/query', method='get', parameters={'q': query})
         while not done:
             full_results.extend(result['records'])
             done = result['done']
@@ -1408,6 +1418,11 @@ class Sf:
     ### ----------------------------------------------------------------------------------------------------
     ### Utility Functions
     ### ----------------------------------------------------------------------------------------------------
+    def query_performance_feedback(self, query: str):
+        # https://developer.salesforce.com/docs/atlas.en-us.240.0.api_rest.meta/api_rest/dome_query_explain.htm
+        
+        return self.sf_api_call(f'/services/data/{self.api_version}/query?explain={query}')
+    
     def parse_sf_limits(self, sfdc_limits_result: dict, prefix=None) -> dict:
         ## Recursively parse the limits result to find the limits that are not 0
         result = {}
@@ -1449,7 +1464,10 @@ class Sf:
             if method=='patch':
                 return None
             else:
-                return r.json()
+                try:
+                    return r.json()
+                except:
+                    return {}
         else:
             raise Exception('API error when calling %s : %s' % (r.url, r.content))
 
